#include <iostream>
using namespace std;

#include <fstream>
#include <sstream>
#include <stdio.h>
#include <stdlib.h>
#include <cstdlib>
#include <unistd.h>
#include <math.h>
#include <sys/time.h>
#include "mines.h"
#include "StdPijo.h"
using namespace STDP;



Stage::Stage(int level){
	Tools tools;
	width=tools.getWidth(level);
	height=tools.getHeight(level);
	nMines=tools.getNMines(level);
	nTeleporters=tools.getNTeleporters(level);
	nSliders=tools.getNSliders(level);
	nBalls=tools.getNBalls(level);
	nCoins=tools.getNCoins(level);
	horizontal=tools.isHorizontal(level);
	movements=0;
	coins=0;
}

void Stage::putCoins(){
	int i, x, y;
	for(i=0;i<nCoins;i++){
		do{
			y = rand() % (height-2) + 1;//TOP_MARGIN;
			x = rand() % (width-2) + 1;//LEFT_MARGIN;
		}while(board[x][y]!=' ');
		coin[i].y = y + TOP_MARGIN;
		coin[i].x = x + LEFT_MARGIN;
		board[x][y]='c';
	}
}

void Stage::putSliders(){
	int i, j, x, y;
	for(i=0;i<nSliders;i++){
		do{
			y = rand() % ((height-2)/2) + height/4 + TOP_MARGIN+1;
			x = rand() % ((width-2)/2) + width/4 + LEFT_MARGIN+1;
		}while(board[x][y]!=' ');
		slider[i].y = y;
		slider[i].x = x;
		for(j=0;j<slider[0].length;j++){
			board[x+j][y]='r'; // ver simbolo
		}
	}
}

void Stage::putTeleporters(){
	int i, x1, y1, x2, y2;
	//nTeleporters=1;
	for(i=0;i<nTeleporters;i++){
		do{
			x1 = rand() % (width-2)/4 + (width-2)/4 + 1;
			y1 = rand() % (height-2) + TOP_MARGIN + 1;
			x2 = rand() % (width-2)/4 + (width-2)/2 + 1;
			y2 = rand() % (height-2) + TOP_MARGIN + 1;
		}while(board[x1][y1]!=' ' || board[x2][y2]!=' ');
		board[x1][y1]='t'; // ver simbolo
		board[x2][y2]='t'; // ver simbolo
		teleporter[i].x1=x1;
		teleporter[i].y1=y1;
		teleporter[i].x2=x2;
		teleporter[i].y2=y2;
	}
}

void Stage::putReservedAreas(){
	int i;
	char symH, symV;
	if(horizontal){
		symH='w';
		symV='g';
	}else{
		symH='g';
		symV='w';
	}
	for(i=0;i<width;i++){
		board[i][0]=symH;
		board[i][height-1]=symH;
	}
	for(i=0;i<height;i++){
		board[0][i]=symV;
		board[width-1][i]=symV;
	}
	if(horizontal){
		reservedAreas.x1=0+LEFT_MARGIN;
		reservedAreas.y1=(height-reservedAreas.text1.length())/2;
		reservedAreas.x2=width+LEFT_MARGIN-1;
		reservedAreas.y2=(height-reservedAreas.text2.length())/2;;
	}else{
		reservedAreas.x1=(width-reservedAreas.text2.length())/2;;
		reservedAreas.y1=height+TOP_MARGIN-1;
		reservedAreas.x2=(width-reservedAreas.text1.length())/2;
		reservedAreas.y2=0+TOP_MARGIN;
	}
}

void Stage::putPlayer(){
	if(horizontal){
		player.x=0;
		player.y=height/2;
	}else{
		player.x=width/2;
		player.y=height;
	}
}

void Stage::clearBoard(){
	int i, j;
	for(i=0;i<width;i++){
		for(j=0;j<height;j++){
			board[i][j]=' ';
		}
	}
}

bool Game::showStage(StdPijo &stdp){
	int i, key;
	double time1, time2, time3, time4;
	bool play=false;
	bool end=false;
	Stage st(level);
	Tools tools;
	timeval tim;

	//Limpia el tablero y coloca los nuevos objetos
	stdp.clearScreen();
	st.clearBoard();
	st.clearMovements();
	st.putReservedAreas();
	st.putSliders();
	st.putTeleporters();
	st.putMines();
	st.putCoins();
	st.putPlayer();
	gettimeofday(&tim, NULL);
	time1=tim.tv_sec+(tim.tv_usec/1000000.0);



	//bucle: mustra el tablero y los movimientos
	do{
		//st.show(stdp);
		st.showMovements(stdp);
		st.showSliders(stdp);
		st.showTeleporters(stdp);

		gettimeofday(&tim, NULL);
		time2=tim.tv_sec+(tim.tv_usec/1000000.0);
		time2=tools.getBonus(level)-(time2-time1)*10;
		if(time2<0)
			time2=0;

		st.showTexts(stdp, level, score, time2);
		st.showReservedAreas(stdp);
		st.showCoins(stdp);
		st.showPlayer(stdp);
		stdp.refreshScr();
		usleep(1000);

				key=stdp.getKey();
				if(st.isMovement(key)){
					st.move(key, stdp);
					//stdp.refreshScr();// ver si ponemos esto aki
					//usleep(500);
					st.efectos(stdp);// poner refrescar pantanlla
					if(st.isWin()){
						st.showWin(stdp);
						end=true;
						play=true;
					}else if(st.isExplosion()){
						st.showExplosion(stdp);
						end=true;
					}else if(st.isMoney()){
						st.addCoin();
						st.hideCoin(stdp);
					}
				}else if(st.isShoot(key)){
					do{
						key=stdp.getKey();
					}while(!tools.isMovement(key));
					st.shoot(stdp, key);
				}else if(key==80 || key==112){
					gettimeofday(&tim, NULL);
					time3=tim.tv_sec+(tim.tv_usec/1000000.0);
					showControls(stdp);
					gettimeofday(&tim, NULL);
					time4=tim.tv_sec+(tim.tv_usec/1000000.0);
					time1=time1+(time4-time3);
				}
	}while(key != KESCAPE && end==false);
	st.showPlayer(stdp);
	stdp.refreshScr();
	fflush(stdin);
	fflush(stdout);
	usleep(5000000);
	if(play){
		score=st.showScore(stdp, level, score, (int)time2);
		//do{
			//key=stdp.getKey();
			//usleep(1000);
		//}while(key != 32);
	}//else
		//st.showGameOver(stdp, level);
		//stdp.refreshScr();
		//do{
			//key=stdp.getKey();
			//usleep(1000);
		//}while(key != KESCAPE);
	return play;
}

bool Game::showGameOver(StdPijo &stdp){
	int key, x, y;
	bool record;
	bool play=true;
	string text;
	Tools tools;
	Menu menu(2);
	stdp.clearScreen();

	x=0;
	y=3;
	stdp.setColor(ABOLD, CWHITE, CBLUE);
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(40, 5);
	stdp.writeStr(text.c_str());
	stdp.writeStr("MISSION ");
	text=tools.intToString(level);
	stdp.writeStr(text.c_str());
	text=tools.completeSpaces(40, 3+tools.intLenght(level));
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());

	do{
		x=30;
		y=10;
		stdp.setColor(ABOLD, CWHITE, CBLACK);
		stdp.setCursor(x, y);
	stdp.writeStr("     GAME OVER      ");
	menu.selectFormat(stdp, 1);
	y=y+2;
	stdp.setCursor(x, y);
	stdp.writeStr("  1. Try it again   ");
	menu.selectFormat(stdp, 2);
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("  2. Surrender      ");
	menu.selectFormat(stdp, 0);
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("                    ");


	key=stdp.getKey();
	if(tools.isUp(key) && menu.active>1){
		menu.active--;
		play=true;
	}else if(tools.isDown(key) && menu.active<menu.options){
		menu.active++;
		play=false;
	}
	usleep(1000);
	}while(key!=32);
	if(play==false && level>1){
		record=actualizarRecords();
		if(record)
			showRanking(stdp);
	}
	return play;
}

int Stage::getPoints(){
	int points;
	points=height*width;
	points=points+coins*50;
	//time
	//otros
	return points;
}

//calcula la posición de las minas de forma aleatoria
void Stage::putMines(){
	int i;
	for(i=0;i<nMines;i++){
		mine[i].y = rand() % (height-2) + TOP_MARGIN+1;
		mine[i].x = rand() % (width-2) + LEFT_MARGIN+1;
	}
}

void Stage::show(StdPijo &stdp)const{
	int i;
	string text;
	Tools tools;

	//muestra cabeceras
	stdp.setCursor(0, 18);
	stdp.writeStr("coordenadas de las minas: ");
	stdp.setCursor(0, 19);
	stdp.writeStr("x: ");
	stdp.setCursor(0, 20);
	stdp.writeStr("y: ");

	for(i=0;i<nMines;i++){

		//muestra las minas
		stdp.setCursor(mine[i].x, mine[i].y);
		stdp.writeChar('*');

		//muestra las coordenadas x de las minas
		stdp.setCursor(3+i*3, 19);
		text=tools.intToString(mine[i].x);
		stdp.writeStr(text.c_str());

		//muestra las coordenadas y de las minas
		stdp.setCursor(3+i*3, 20);
		text=tools.intToString(mine[i].y);
		stdp.writeStr(text.c_str());
	}

	//muestra los movimientos
	stdp.setCursor(0, 16);
	stdp.writeStr("Movimientos: ");
	text=tools.intToString(movements);
	stdp.writeStr(text.c_str());
}

void Stage::showCoins(StdPijo &stdp){
	int i;
	stdp.setColor(ABOLD, CYELLOW, CBLACK);
	for(i=0;i<nCoins;i++){
		stdp.setCursor(coin[i].x, coin[i].y);
		stdp.writeChar('o');
	}
}

void Stage::showSliders(StdPijo &stdp){ // ver porque siempre sale al lado de t
	int i, k;
	//nSliders=1;
	slider[0].length=5;
	stdp.setColor(ABOLD, CWHITE, CBLUE);
	for(i=0;i<nSliders;i++){
		stdp.setCursor(slider[i].x, slider[i].y);
		for(k=0;k<slider[0].length;k++){
			//stdp.setCursor(slider[i].x+k, slider[i].y);
			stdp.writeChar('>');//ver variable slider[i].sense
		}
	}
}

void Stage::showReservedAreas(StdPijo &stdp){
	int i, j;
	string text;
	for(i=0;i<width;i++){
		for(j=0;j<height;j++){
			if(board[i][j]=='g'){
				stdp.setColor(ABOLD, CWHITE, CGREEN);
				stdp.setCursor(i+LEFT_MARGIN, j+TOP_MARGIN);
				stdp.writeChar(' ');
			}else if(board[i][j]=='w'){
				stdp.setColor(ABOLD, CWHITE, CWHITE);
				stdp.setCursor(i+LEFT_MARGIN, j+TOP_MARGIN);
				stdp.writeChar(' ');
			}
		}
	}
	stdp.setColor(ABOLD, CWHITE, CGREEN);
	if(horizontal){
		for(i=0;i<reservedAreas.text1.length();i++){
			stdp.setCursor(reservedAreas.x1, reservedAreas.y1+i);
			stdp.writeChar(reservedAreas.text1[i]);
		}
		for(i=0;i<reservedAreas.text2.length();i++){
			stdp.setCursor(reservedAreas.x2, reservedAreas.y2+i);
			stdp.writeChar(reservedAreas.text2[i]);
		}
	}else{
		stdp.setCursor(reservedAreas.x1, reservedAreas.y1);
		stdp.writeStr(reservedAreas.text1.c_str());
		stdp.setCursor(reservedAreas.x2, reservedAreas.y2);
		stdp.writeStr(reservedAreas.text2.c_str());
	}
}

void Stage::showMines(StdPijo &stdp, bool kind){
	int i;
	if(kind==true)
		stdp.setColor(ABOLD, CGREEN, CBLACK);
	else
		stdp.setColor(ABOLD, CRED, CBLACK);
	for(i=0;i<nMines;i++){
		stdp.setCursor(mine[i].x, mine[i].y);
		stdp.writeChar(mine[i].sym);
	}
}

void Stage::showTeleporters(StdPijo &stdp){
	int i;
	stdp.setColor(ABOLD, CBLUE, CBLACK);
	for(i=0;i<nTeleporters;i++){
		stdp.setCursor(teleporter[i].x1, teleporter[i].y1);
		stdp.writeChar('t');
		stdp.setCursor(teleporter[i].x2, teleporter[i].y2);
		stdp.writeChar('t');
	}
}

void Stage::showPlayer(StdPijo &stdp){
    stdp.setColor(ABOLD, CYELLOW, CBLACK);
    stdp.setCursor(player.x, player.y);
    stdp.writeChar(player.sym);
}


void Stage::showTexts(StdPijo &stdp, int level, int score, int time){
	int i;
	string text;
	Tools tools;
	stdp.setCursor(0, 0);
	stdp.setColor(ABOLD, CWHITE, CBLACK);
	stdp.writeStr("                    ");
	stdp.writeStr("                    ");
	stdp.writeStr("                    ");
	stdp.writeStr("                    ");
	stdp.setCursor(2, 0);
	stdp.setColor(ABOLD, CWHITE, CBLACK);
	stdp.writeStr("Level: ");
	stdp.setColor(ABOLD, CYELLOW, CBLACK);
	text=tools.intToString(level);
	stdp.writeStr(text.c_str());
	stdp.setColor(ABOLD, CWHITE, CBLACK);
	stdp.writeStr(" Coins: ");
	stdp.setColor(ABOLD, CYELLOW, CBLACK);
	text=tools.intToString(coins);
	stdp.writeStr(text.c_str());
	stdp.setColor(ABOLD, CWHITE, CBLACK);
	stdp.writeStr(" Balls: ");
	stdp.setColor(ABOLD, CYELLOW, CBLACK);
	text=tools.intToString(nBalls);
	stdp.writeStr(text.c_str());
	stdp.setColor(ABOLD, CWHITE, CBLACK);
	stdp.writeStr(" Bonus: ");
	stdp.setColor(ABOLD, CYELLOW, CBLACK);
	text=tools.intToString(time);
	stdp.writeStr(text.c_str());

	//parte de abajo
	stdp.setCursor(2, height+TOP_MARGIN+1);
	stdp.setColor(ABOLD, CWHITE, CBLACK);
	stdp.writeStr(" Mines near: ");
	if(countMines()==0)
		stdp.setColor(ABOLD, CWHITE, CGREEN);
	else
		stdp.setColor(ABOLD, CWHITE, CRED);
	stdp.writeChar(' ');
	stdp.writeChar(countMines()+48);
	stdp.writeChar(' ');


	stdp.setCursor(23, 22);
	stdp.setColor(ABOLD, CMAGENTA, CBLACK);
	stdp.writeStr("Press [P] key to pause or to help");
}

bool Stage::isWin(){
	bool active=false;
	if(horizontal){
		if(player.x==width-1+LEFT_MARGIN)
			active=true;
	}else{
		if(player.y==0+TOP_MARGIN)
			active=true;
	}
	return active;
}

void Stage::showWin(StdPijo &stdp){
	showMines(stdp, true);
}

bool Stage::isShoot(int key){
	bool active=false;
	if(nBalls>0 && (key==66 || key==98))
			active=true;
	return active;
}

void Stage::shoot(StdPijo &stdp, int key){
	int i;
	int numMine=-1;
	bool end=false;
	Tools tools;
	ball.x=player.x;
	ball.y=player.y;
	do{
		hideBall(stdp);
		if(tools.isUp(key)){
	    	ball.y--;
	    }
	    if(tools.isDown(key)){
	    	ball.y++;
	    }
	    if(tools.isLeft(key)){
	    	ball.x--;
	    }
	    if(tools.isRight(key)){
	       	ball.x++;
	    }
	    showBall(stdp);
	    showPlayer(stdp);
	    end=endBall();
	    if(!end){
	    	numMine=explosionBall();
	    	if(numMine>=0)
	    		deleteMine(stdp, numMine);
	    	 stdp.refreshScr();
	    	 usleep(100000);
	    }
	}while(numMine<0 && end==false);
	nBalls--;
}

void Stage::deleteMine(StdPijo &stdp, int numMine){
	stdp.setColor(ABOLD, CRED, CBLACK);
	stdp.setCursor(mine[numMine].x, mine[numMine].y);
	stdp.writeChar('*');
	stdp.refreshScr();
	usleep(100000);
	stdp.setColor(ABOLD, CBLUE, CBLACK);
	stdp.setCursor(mine[numMine].x, mine[numMine].y);
	stdp.writeChar(254);
	mine[numMine]=mine[nMines-1];
	nMines--;
}

bool Stage::endBall(){
	bool end;
	if(ball.x==0+LEFT_MARGIN || ball.x==width+LEFT_MARGIN-1)
		end=true;
	else if(ball.y==0+TOP_MARGIN || ball.y==height+TOP_MARGIN-1)
		end=true;
	return end;
}

int Stage::explosionBall(){
	int i;
	int num=-1;
    for(i=0;i<nMines;i++){
    	if(mine[i].x==ball.x && mine[i].y==ball.y)
    		num=i;
    }
    return num;
}

void Stage::showBall(StdPijo &stdp){
	stdp.setColor(ABOLD, CBLUE, CBLACK);
	stdp.setCursor(ball.x, ball.y);
	stdp.writeChar('.');
}

void Stage::hideBall(StdPijo &stdp){
	stdp.setColor(ABOLD, CBLUE, CBLACK);
	stdp.setCursor(ball.x, ball.y);
	stdp.writeChar(' ');
}

bool Stage::isMoney(){
	bool active=false;
	if(board[player.x-LEFT_MARGIN][player.y-TOP_MARGIN]=='c')
		active=true;
	return active;
}

void Stage::addCoin(){
	coins++;
}
void Stage::hideCoin(StdPijo &stdp){
	int i;
	board[player.x-LEFT_MARGIN][player.y-TOP_MARGIN]=' ';
	stdp.setCursor(player.x, player.y);
	stdp.setColor(ABOLD, CWHITE, CBLACK);
	for(i=0;i<nCoins;i++){
		if(coin[i].x==player.x && coin[i].y==player.y)
			coin[i]=coin[nCoins-1];
	}
	nCoins--;
}



void Stage::efectos(StdPijo &stdp){
	int i;
	if(board[player.x][player.y]=='t'){
		for(i=0;i<nTeleporters;i++){
			if(teleporter[i].x1==player.x && teleporter[i].y1==player.y){
				player.y=teleporter[i].y2;
				player.x=teleporter[i].x2;
			}else if(teleporter[i].x2==player.x && teleporter[i].y2==player.y){
				player.y=teleporter[i].y1;
				player.x=teleporter[i].x1;
			}
		}
	}else if(board[player.x][player.y]=='r'){
		do{
			showSliders(stdp);
			player.x++;
			showPlayer(stdp);
			stdp.refreshScr();
			usleep(100000);
		}while(board[player.x][player.y]=='r');
	}
}

void Stage::showExplosion(StdPijo &stdp){
	showMines(stdp, false);
}

int Stage::countMines(){ //TENER ENCUENTA QUE HAY QUE PONER PRIMERO isExplosion
	int i,j,k;
	int amount=0;
	int PosX=player.x-1;
	int PosY=player.y-1;
	for(i=0;i<3;i++){
		for(j=0;j<3;j++){
			for(k=0;k<nMines;k++){
				if(mine[k].x==PosX && mine[k].y==PosY)
					amount++;
			}
		PosX++;
		}
		PosY++;
		PosX=PosX-3;
	}
	return amount;
}

bool Stage::isExplosion(){
	int i;
	bool active=false;
	for(i=0;i<nMines;i++){
		if(mine[i].x==player.x && mine[i].y==player.y)
			active=true;
	}
	return active;
}

bool Stage::isMovement(int key){
	bool active=false;
	Tools tools;
	if(tools.isUp(key) && player.y>TOP_MARGIN+horizontal)
		active=true;
	if(tools.isDown(key) && player.y<height-horizontal)
		active=true;
	if(tools.isLeft(key) && player.x>LEFT_MARGIN-horizontal+1)
		active=true;
	if(tools.isRight(key) && player.x<width+horizontal-2)
		active=true;
	return active;
}

void Stage::clearMovements(){
	int i, j;
	for(i=0;i<MAX_WIDTH;i++){
		for(j=0;j<MAX_HEIGHT;j++){
			mov[i][j]=' ';
		}
	}
}

void Stage::showMovements(StdPijo &stdp){
	int i, j;
	for(i=0;i<MAX_WIDTH;i++){
		for(j=0;j<MAX_HEIGHT;j++){
			if(mov[i][j]=='<' || mov[i][j]=='^' || mov[i][j]=='>'|| mov[i][j]=='v')
				stdp.setColor(ABOLD, CWHITE, CBLACK);
				stdp.setCursor(i, j);
				stdp.writeChar(mov[i][j]);
		}
	}
}

void Stage::move(int key, StdPijo &stdp){
	Tools tools;
    stdp.setColor(ABOLD, CWHITE, CBLACK);
    stdp.setCursor(player.x, player.y);
	if(tools.isUp(key)){
        stdp.writeChar('^');
        mov[player.x][player.y]='^';
    	player.y--;
    }else if(tools.isDown(key)){
        stdp.writeChar('v');
        mov[player.x][player.y]='v';
    	player.y++;
    }else if(tools.isLeft(key)){
        stdp.writeChar('<');
        mov[player.x][player.y]='<';
    	player.x--;
    }else if(tools.isRight(key)){
        stdp.writeChar('>');
        mov[player.x][player.y]='>';
       	player.x++;
    }
 	movements++;
}


string Tools::intToString(int number)const{
	stringstream ss;
	ss.str("");
	ss << number;
	return (ss.str());
}

void Game::addLevel(){
	level++;
}

void Menu::titleFormat(StdPijo &stdp){
	stdp.setColor(ABOLD, CWHITE, CBLACK);
}

void Menu::normalFormat(StdPijo &stdp){
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
}

void Menu::activeFormat(StdPijo &stdp){
	stdp.setColor(ABOLD, CWHITE, CBLUE);
}

void Menu::selectFormat(StdPijo &stdp, int option){
	if(option==0)
		normalFormat(stdp);
	else if(active==option)
		activeFormat(stdp);
	else
		normalFormat(stdp);
}

int Menu::show(StdPijo &stdp){
	int x, y, key;
	string text;
	Tools tools;
	do{
		x=0;
		y=3;
	stdp.setColor(ABOLD, CWHITE, CBLUE);
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());

	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(40, 5);
	stdp.writeStr(text.c_str());
	stdp.writeStr("MAIN MENU");
	text=tools.completeSpaces(40, 4);
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());


	y=y+4;
	x=20;
	selectFormat(stdp, 999);
	stdp.setCursor(x, y);
	text=tools.completeSpaces(40, 0);
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	selectFormat(stdp, 1);
	stdp.writeStr("          1. PLAY                       ");
	y++;
	selectFormat(stdp, 2);
	stdp.setCursor(x, y);
	stdp.writeStr("          2. CONTROLS                   ");
	y++;
	selectFormat(stdp, 3);
	stdp.setCursor(x, y);
	stdp.writeStr("          3. RANKING                    ");
	y++;
	stdp.setCursor(x, y);
	selectFormat(stdp, 4);
	stdp.writeStr("          4. EXIT                       ");
	//y++;
	//stdp.setCursor(x, y);
	//selectFormat(stdp, 5);
	//stdp.writeStr("          5. EXIT                       ");
	y++;
	selectFormat(stdp, 999);
		stdp.setCursor(x, y);
		text=tools.completeSpaces(40, 0);
		stdp.writeStr(text.c_str());

	key=stdp.getKey();
	if(tools.isUp(key) && active>1)
		active--;
	else if(tools.isDown(key) && active<options)
		active++;
	usleep(1000);
	}while(key!=32);
	return active;
}

int Game::showMenu(StdPijo &stdp){
	int option;
	Menu menu(4);
	stdp.clearScreen();
	option=menu.show(stdp);
	return option;
}

void StageMaker::clearBoard(char board[][20]){
	int i, j;
	for(i=0;i<width;i++){
		for(j=0;j<height;j++){
			if(i==0 || i==width-1)
				board[i][j]='g';
			else if(j==0 || j==height-1)
				board[i][j]='w';
			else
				board[i][j]=' ';
		}
	}
}

void Game::showStageMaker(StdPijo &stdp){
	int x, y, key;
	char board[80][20];
	StageMaker stm;
	stdp.clearScreen();
	stm.clearBoard(board);
	do{
		key=stdp.getKey();
		if(stm.isMovement(key))
			stm.movement(key);
		else if(stm.isDraw(key))
			stm.draw(board, key);
		stm.showBoard(stdp, board);
		stm.showCursor(stdp);
		stdp.refreshScr();
		usleep(1000);
	}while(key!=KESCAPE);
}

void StageMaker::showCursor(StdPijo &stdp){
	stdp.setColor(ABLINK, CWHITE, CRED);
	stdp.setCursor(cursor.x, cursor.y);
	stdp.writeChar(' ');
}

void StageMaker::draw(char board[][20], int key){
	board[cursor.x-LEFT_MARGIN][cursor.y-TOP_MARGIN]= (char)key;
	movement(KRIGHT2);
}

bool StageMaker::isDraw(int key){
	bool draw=false;
	if(key=='*' || key=='o' || key=='t' || key=='r' || key==' ')
		draw=true;
	else if(key=='*' || key=='O' || key=='T' || key=='R' || key==' ')
		draw=true;
	return draw;
}

void StageMaker::showBoard(StdPijo &stdp, char board[][20]){
	int i, j;
	char sym;
	for(i=0;i<height;i++){
		stdp.setCursor(0, i+1);
		for(j=0;j<width;j++){
			sym=getSym(board[j][i]);
			symFormat(stdp, sym);
			stdp.writeChar(sym);
		}
	}
}

char StageMaker::getSym(char sym){
	char newSym;
	if(sym=='*' || sym=='t' || sym=='o' || sym=='w' || sym=='g')
		newSym=sym;
	else if(sym='r')
		newSym='>';
	else
		newSym=32;
	return newSym;
}

void StageMaker::symFormat(StdPijo &stdp,char sym){
	if(sym=='>')
		stdp.setColor(ABOLD, CWHITE, CBLUE);
	else if(sym=='t')
		stdp.setColor(ABOLD, CWHITE, CBLUE);
	else if(sym=='g')
		stdp.setColor(ANORMAL, CGREEN, CGREEN);
	else if(sym=='w')
		stdp.setColor(ANORMAL, CWHITE, CWHITE);
	else if(sym=='o')
		stdp.setColor(ABOLD, CYELLOW, CBLACK);
	else if(sym=='*')
		stdp.setColor(ABOLD, CRED, CBLACK);
	else
		stdp.setColor(ABOLD, CRED, CBLACK);
}

bool StageMaker::isMovement(int key){
	bool active=false;
	Tools tools;
	if(tools.isUp(key) && cursor.y>TOP_MARGIN+1)
		active=true;
	if(tools.isDown(key) && cursor.y<height-1)
		active=true;
	if(tools.isLeft(key) && cursor.x>LEFT_MARGIN)
		active=true;
	if(tools.isRight(key) && cursor.x<width-1)
		active=true;
	return active;
}

void StageMaker::movement(int key){
	Tools tools;
	if(tools.isUp(key))
	   	cursor.y--;
	if(tools.isDown(key))
	   	cursor.y++;
	if(tools.isLeft(key))
	 	cursor.x--;
	if(tools.isRight(key)){
		if(cursor.x<width-1+LEFT_MARGIN)
			cursor.x++;
		else if(cursor.y<height-1+TOP_MARGIN){
			cursor.x=0;
			cursor.y++;
		}
	}
}

int Stage::showScore(StdPijo &stdp, int level, int score, int bonus){
	int key, points;
	int x=0;
	int y=3;
	string text;
	stringstream sText;
	Tools tools;
	Level sl(level);
	stdp.clearScreen();
	stdp.setColor(ABOLD, CWHITE, CBLUE);
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(40, 10);
	stdp.writeStr(text.c_str());
	stdp.writeStr("CONGRATULATIONS!!!!!");
	text=tools.completeSpaces(40, 10);
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());


	x=18;
	y=y+3;
	stdp.setCursor(x-5, y);
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Previous Score: ");

	stdp.setCursor(x+11, y);
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	text=tools.intToString(score);
	sText.str("");
	sText << text << " $";
	stdp.writeStr((sText.str()).c_str());

	y=y+3;
	stdp.setCursor(x, y);
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Wage: ");
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	stdp.setCursor(x+11, y);
	text=tools.intToString(tools.getWage(level));
	sText.str("");
	sText << text << " $";
	stdp.writeStr((sText.str()).c_str());

	y++;
	stdp.setCursor(x, y);
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Bonus: ");
	stdp.setCursor(x+11, y);
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	text=tools.intToString(bonus);
	sText.str("");
	sText << text << " $";
	stdp.writeStr((sText.str()).c_str());

	y++;
	if(tools.getNCoins(level)>0){
		stdp.setCursor(x, y);
		stdp.setColor(ANORMAL, CWHITE, CBLACK);
		stdp.writeStr("Coins: ");
		stdp.setCursor(x+11, y);
		stdp.setColor(ANORMAL, CYELLOW, CBLACK);
		text=tools.intToString(coins*50);
		sText.str("");
		sText << text << " $";
		stdp.writeStr((sText.str()).c_str());
	}

	y=y+3;
	stdp.setCursor(x-5, y);
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("  Final Score: ");
	stdp.setCursor(x+11, y);
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	points=score+tools.getWage(level)+bonus+coins*50;
	text=tools.intToString(points);
	sText.str("");
	sText << text << " $";
	stdp.writeStr((sText.str()).c_str());


	stdp.setCursor(25, 19);
	sl.showPressAnyKey(stdp);
	do{
		key=stdp.getKey();
		usleep(1000);
	}while(key!=32);
	return points;
}

void Game::showLevel(StdPijo &stdp){
	int key, num;
	int x=5;
	int y=3;
	string text;
	stringstream sText;
	Tools tools;
	Level sl(level);
	stdp.clearScreen();
	stdp.setCursor(x, y);
	sl.showTitle(stdp);
	y=y+5;
	stdp.setCursor(x, y);
	sl.showDescription(stdp);
	stdp.setCursor(38, y);
	sl.showBenefits(stdp);
	y=y+2;
	x=x+7;
	stdp.setCursor(x, y);
	sl.showSize(stdp);
	stdp.setCursor(43, y);
	sl.showWage(stdp);
	y++;
	stdp.setCursor(x, y);
	sl.showNMines(stdp);
	stdp.setCursor(43, y);
	sl.showBonus(stdp);
	y++;
	stdp.setCursor(x, y);
	sl.showNCoins(stdp);
	if(tools.getNCoins(level)>0){
		stdp.setCursor(43, y);
		sl.showPriceCoin(stdp);
	}
	y++;
	stdp.setCursor(x, y);
	sl.showNBalls(stdp);
	y++;
	stdp.setCursor(x, y);
	sl.showNTeleporters(stdp);
	y++;
	stdp.setCursor(x, y);
	sl.showNSliders(stdp);


	stdp.setCursor(25, 18);
	sl.showPressAnyKey(stdp);
	key=-1;
	do{
		key=stdp.getKey();
		usleep(1000);
	}while(key<0);
}

void Level::showTitle(StdPijo &stdp){
	string text;
	Tools tools;
	int x=0;
	int y=3;
	stdp.setColor(ABOLD, CWHITE, CBLUE);
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(40, 5);
	stdp.writeStr(text.c_str());
	stdp.writeStr("MISSION ");
	text=tools.intToString(level);
	stdp.writeStr(text.c_str());
	text=tools.completeSpaces(40, 3+tools.intLenght(level));
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());
}

void Level::showDescription(StdPijo &stdp){
	stdp.setColor(ABOLD, CWHITE, CBLACK);
	stdp.writeStr("       DESCRIPTION:       ");
}

void Level::showBenefits(StdPijo &stdp){
	stdp.setColor(ABOLD, CWHITE, CBLACK);
	stdp.writeStr("     BENEFITS:  ");
}


void Level::showSize(StdPijo &stdp){
	Tools tools;
	stringstream sText;
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Size: ");
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	sText.str("");
	sText << tools.getWidth(level) << "x" << tools.getHeight(level);
	stdp.writeStr((sText.str()).c_str());
}

void Level::showWage(StdPijo &stdp){
	Tools tools;
	string text;
	stringstream sText;
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Wage: ");
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	text=tools.intToString(tools.getWage(level));
	sText.str("");
	sText << text << " $";
	stdp.writeStr((sText.str()).c_str());
}

void Level::showNMines(StdPijo &stdp){
	Tools tools;
	string text;
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Mines: ");
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	text=tools.intToString(tools.getNMines(level));
	stdp.writeStr(text.c_str());
}
void Level::showBonus(StdPijo &stdp){
	Tools tools;
	string text;
	stringstream sText;
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Bonus: ");
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	text=tools.intToString(tools.getBonus(level));
	sText.str("");
	sText << text << " $";
	stdp.writeStr((sText.str()).c_str());
}

void Level::showNCoins(StdPijo &stdp){
	Tools tools;
	string text;
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Coins: ");
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	text=tools.intToString(tools.getNCoins(level));
	stdp.writeStr(text.c_str());
}


void Level::showPriceCoin(StdPijo &stdp){
	Tools tools;
	string text;
	stringstream sText;
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Each coin: ");
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	stdp.writeStr("50 $");
}


void Level::showNBalls(StdPijo &stdp){
	Tools tools;
	string text;
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Balls: ");
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	text=tools.intToString(tools.getNBalls(level));
	stdp.writeStr(text.c_str());
}

void Level::showNTeleporters(StdPijo &stdp){
	Tools tools;
	string text;
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Teleporters: ");
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	text=tools.intToString(tools.getNTeleporters(level));
	stdp.writeStr(text.c_str());
}

void Level::showNSliders(StdPijo &stdp){
	Tools tools;
	string text;
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.writeStr("Sliders: ");
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	text=tools.intToString(tools.getNSliders(level));
	stdp.writeStr(text.c_str());
}


void Level::showPressAnyKey(StdPijo &stdp){
	stdp.setColor(ABOLD, CMAGENTA, CBLACK);
	stdp.writeStr("Press [Space] key to continue");
}

int Tools::getWidth(int level){
	int width=21+level;
	if(width>MAX_WIDTH-LEFT_MARGIN)
		width=MAX_WIDTH-LEFT_MARGIN;
	return width;
}

int Tools::getHeight(int level){
	int height=11+level;
	if(height>MAX_HEIGHT-TOP_MARGIN)
		height=MAX_HEIGHT-TOP_MARGIN;
	return height;
}

int Tools::getNMines(int level){
	int nMines=level*5;
	return nMines;
}

bool Tools::isHorizontal(int level){
	bool horizontal=false;
	if(level>10)
		horizontal=true;
	return horizontal;
}

int Tools::getNTeleporters(int level){
	int nTeleporters=level/15;
	return nTeleporters;
}

int Tools::getNSliders(int level){
	int nSliders=(level/20);
	return nSliders;
}

int Tools::getNBalls(int level){
	int nBalls=level/10;
	return nBalls;
}

int Tools::getNCoins(int level){
	int nCoins=2*(level/5);
	return nCoins;
}

int Tools::getBonus(int level){
	int bonus=level*200+300;
	return bonus;
}

int Tools::getWage(int level){
	int wage=getHeight(level)*getWidth(level);
	return wage;
}

bool Tools::isUp(int key){
	bool active=false;
	if(key==KUP || key==KUP2 || key==KUP3)
		active=true;
	return active;
}

bool Tools::isDown(int key){
	bool active=false;
	if(key==KDOWN || key==KDOWN2 || key==KDOWN3)
		active=true;
	return active;
}

bool Tools::isLeft(int key){
	bool active=false;
	if(key==KLEFT || key==KLEFT2 || key==KLEFT3)
		active=true;
	return active;
}

bool Tools::isRight(int key){
	bool active=false;
	if(key==KRIGHT || key==KRIGHT2 || key==KRIGHT3)
		active=true;
	return active;
}

bool Tools::isMovement(int key){
	bool active=false;
	if(isUp(key) || isDown(key) || isLeft(key) || isRight(key))
		active=true;
	return active;

}

void Game::showRanking(StdPijo &stdp){
	int i, key;
	string text;
	stringstream sText;
	Tools tools;
	int x=0;
	int y=2;
	stdp.clearScreen();
	stdp.setColor(ABOLD, CWHITE, CBLUE);
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(40, 4);
	stdp.writeStr(text.c_str());
	stdp.writeStr("RANKING");
	text=tools.completeSpaces(40, 3);
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());
	y=y+3;
	x=10;
	stdp.setCursor(x, y);
	stdp.setColor(ABOLD, CWHITE, CGREEN);
	stdp.writeStr("       Rank         Points       Level          Date         ");


	for(i=1;i<=10;i++){
		stdp.setColor(AUNDERLINE, CBLACK, CWHITE);
		y++;
		stdp.setCursor(x, y);
		sText<<i;
		text=tools.completeSpaces(10, tools.intLenght(i));
		stdp.writeStr(text.c_str());
		text=tools.intToString(i);
		stdp.writeStr(text.c_str());
		text=tools.completeSpaces(16, tools.intLenght(records[i-1][0]));
		stdp.writeStr(text.c_str());
		text=tools.intToString(records[i-1][0]);
		stdp.writeStr(text.c_str());

		text=tools.completeSpaces(11, tools.intLenght(records[i-1][1]));
		stdp.writeStr(text.c_str());
		text=tools.intToString(records[i-1][1]);
		stdp.writeStr(text.c_str());



		stdp.writeStr("        ");
		if(records[i-1][2]!=0){
			sText.str("");
			if(records[i-1][2]<10)
				sText<<"0";
			sText << records[i-1][2] << "/";
			if(records[i-1][3]<10)
				sText << "0";
			sText << records[i-1][3] << "/" << records[i-1][4];
			stdp.writeStr((sText.str()).c_str());
		}else
			stdp.writeStr("          ");
		stdp.writeStr("      ");

	}

	stdp.setCursor(25, 20);
	stdp.setColor(ABOLD, CMAGENTA, CBLACK);
	stdp.writeStr("Press [Space] key to continue");
	do{
		key=stdp.getKey();
		usleep(1000);
	}while(key!=32);
}


void Game::cargarRecords(){
	int i=0;
	string file="puntuaciones";
	string line;
	ifstream fi;
	fi.open(file.c_str(), ios::in);
	if (fi.is_open()){
		getline(fi,line);
		while (!fi.eof()){
			records[i][0]=extraerNumero(line, 1);
			records[i][1]=extraerNumero(line, 2);
			records[i][2]=extraerNumero(line, 3);
			records[i][3]=extraerNumero(line, 4);
			records[i][4]=extraerNumero(line, 5);
			i++;
			getline(fi,line);
		}
		fi.close();
	}
}


//bool Game::actualizarRecords(int puntos, int tiempo, int categoria){
bool Game::actualizarRecords(){
	Tools tools;
	bool encontrado=false;
	int i;
	int pos=-1;
	int fin=9;
	int ini=fin-9;
	if(records[fin][0]<score || (records[fin][0]==score && records[fin][1]<level)){
		for(i=(fin-1);i>=ini;i--){
			if(records[i][0]<score){
				records[i+1][0]=records[i][0];
				records[i+1][1]=records[i][1];
				records[i+1][2]=records[i][2];
				records[i+1][3]=records[i][3];
				records[i+1][4]=records[i][4];
			}else if(records[i][0]==score && records[i][1]<level){
				records[i+1][0]=records[i][0];
				records[i+1][1]=records[i][1];
				records[i+1][2]=records[i][2];
				records[i+1][3]=records[i][3];
				records[i+1][4]=records[i][4];
			}else if(encontrado==false){
				encontrado=true;
				pos=i;
			}
		}
		if(pos<0)
			pos=ini-1;
		records[pos+1][0]=score;
		records[pos+1][1]=level;
		records[pos+1][2]=tools.getDay();
		records[pos+1][3]=tools.getMonth();
		records[pos+1][4]=tools.getYear();
		grabarRecords();
	}
	return encontrado;
}



void Game::grabarRecords(){
	int i, j;
	string line="";
	ofstream fo;
	fo.open("puntuaciones", ios::out);
	if (fo.is_open()){
		for(i=0;i<10;i++){
				fo << '0' << '#'<< records[i][0] << '#' << records[i][1] << '#' << records[i][2] << '#' << records[i][3] << '#' << records[i][4];
			fo << endl;
		}
		fo.close();
	}
}


string Game::extraerCadena(string cadena, int pos){
	int i;
	int unsigned ini=0;
	int unsigned end=0;
		for (i=0;i<=pos;i++){
			if(end!=cadena.length()){
				if(end!=0)
					ini=end+1;
				if(cadena.find('#', ini)==string::npos)
					end=cadena.length();
				else
					end=cadena.find('#', ini);
			}else
				cadena="";
		}
	if(cadena!="")
			cadena=cadena.substr(ini, end-ini);
	return(cadena);
}

int Game::extraerNumero(string cadena, int pos){
	int num;
	string texto;
	texto=extraerCadena(cadena, pos);
	num=atoi(texto.c_str());
	return num;
}

int Tools::intLenght(int num){
	int lenght=0;
	if(num==0)
		lenght=1;
	else{
		while(num!=0){
			num=num/10;
			lenght++;
		}
	}
	return lenght;
}

string Tools::completeSpaces(int total, int used){
	int i;
	stringstream text;
	for(i=0;i<total-used;i++){
		text << " ";
	}
	return text.str();
}

int Tools::getDay(){
	int day;
	time_t t= time(NULL);
	tm lt = *localtime(&t);
	day=lt.tm_mday;
	return day;
}

int Tools::getMonth(){
	int month;
	time_t t= time(NULL);
	tm lt = *localtime(&t);
	month=lt.tm_mon+1;
	return month;
}

int Tools::getYear(){
	int year;
	time_t t= time(NULL);
	tm lt = *localtime(&t);
	year=lt.tm_year+1900;
	return year;
}
void Game::resetValues(){
	level=1;
	score=0;
}

void Game::showControls(StdPijo &stdp){
	int x, y, key, num;
	string text;
	stringstream sText;
	Tools tools;
	Level sl(level);
	stdp.clearScreen();

	x=0;
	y=3;
	stdp.setColor(ABOLD, CWHITE, CBLUE);
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(40, 4);
	stdp.writeStr(text.c_str());
	stdp.writeStr("CONTROLS");
	text=tools.completeSpaces(40, 4);
	stdp.writeStr(text.c_str());
	y++;
	stdp.setCursor(x, y);
	text=tools.completeSpaces(80, 0);
	stdp.writeStr(text.c_str());
	y=9;
	x=30;
	stdp.setColor(ANORMAL, CWHITE, CBLACK);
	stdp.setCursor(x, y);
	stdp.writeStr("Up: ");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("Down: ");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("Left: ");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("Right: ");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("Back/Exit: ");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("Accept: ");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("Use ball: ");
	y=9;
	x=42;
	stdp.setColor(ANORMAL, CYELLOW, CBLACK);
	stdp.setCursor(x, y);
	stdp.writeStr("W");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("S");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("A");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("D");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("[Esc]");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("[Spc]");
	y++;
	stdp.setCursor(x, y);
	stdp.writeStr("B + Direction");

	stdp.setCursor(25, 18);
	sl.showPressAnyKey(stdp);
	key=-1;
	do{
		key=stdp.getKey();
		usleep(1000);
	}while(key!=32);
	stdp.clearScreen();
}


int main(void){
	StdPijo& stdp = StdPijo::p();
	stdp.init();
	stdp.clearScreen();
	stdp.setGlobalEscapeDelay(0);
	srand(time(NULL));

	int key, option;
	bool play;
	Game mines;
	mines.cargarRecords();

	option=mines.showMenu(stdp);
	while(option!=4){
		if(option==1){
			do{
				mines.showLevel(stdp);
				play=mines.showStage(stdp);
				if(play)
					mines.addLevel();
				else
					play=mines.showGameOver(stdp);
			}while(play);
		}else if(option==2){
			mines.showControls(stdp);
		}else if(option==3){
			mines.showRanking(stdp);
		}//else if(option==4)
			//mines.showStageMaker(stdp);
		mines.resetValues();
		option=mines.showMenu(stdp);
	}
	stdp.end();
}
